package com.caimei365.manager.utils;

import com.github.junrar.Archive;
import com.github.junrar.rarfile.FileHeader;
import lombok.extern.slf4j.Slf4j;
import net.lingala.zip4j.core.ZipFile;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.io.IOUtils;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.*;
import java.nio.charset.Charset;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * Description
 *
 * @author : hzg
 * @date : 2023/12/19
 */
@Slf4j
public class FileUtil {

    /**
     * 将file 文件转为 MultipartFile 文件
     * @param file
     * @return
     * @throws IOException
     */
    public static MultipartFile fileToMultipartFileConverter(File file) throws IOException {
        DiskFileItem diskFileItem = new DiskFileItem("File", "application/octet-stream", true, file.getName(), (int) file.length(), file.getParentFile());
        try(InputStream is = new FileInputStream(file)) {
            diskFileItem.getOutputStream().write(IOUtils.toByteArray(is));
        }
        return new CommonsMultipartFile(diskFileItem);
    }



    /**
     * zip文件解压
     *
     * @param destPath 解压文件路径
     * @param zipFilePath  压缩文件路径
     */
    public static void unZip(String zipFilePath, String destPath) throws IOException {
        File destDir = new File(destPath);
        if (!destDir.exists()) {
            destDir.mkdir();
        }
        ZipInputStream zipIn = new ZipInputStream(new FileInputStream(zipFilePath), Charset.forName("GBK"));
        ZipEntry entry = zipIn.getNextEntry();
        while (entry != null) {
            String filePath = destPath + File.separator + entry.getName();
            if (!entry.isDirectory()) {
                File file = new File(filePath);
                file.getParentFile().mkdirs();
                try (FileOutputStream fos = new FileOutputStream(file)) {
                    byte[] bytes = new byte[4096];
                    int length;
                    while ((length = zipIn.read(bytes)) != -1) {
                        fos.write(bytes, 0, length);
                    }
                }
            } else {
                File dir = new File(filePath);
                dir.mkdir();
            }
            zipIn.closeEntry();
            entry = zipIn.getNextEntry();
        }
        zipIn.close();
    }

    /**
     * zip文件解压
     *
     * @param destPath 解压文件路径
     * @param zipFile  压缩文件
     */
    public static void unPackZip(File zipFile, String destPath) {
        try {
            ZipFile zip = new ZipFile(zipFile);
            /*zip4j默认用GBK编码去解压,这里设置编码为GBK的*/
            zip.setFileNameCharset("GBK");
            log.info("begin unpack zip file....");
            zip.extractAll(destPath);
        } catch (Exception e) {
            log.error("解压失败：", e.getMessage(), e);
        }
    }

    /**
     * rar文件解压(不支持有密码的压缩包)
     *
     * @param rarFile  rar压缩包
     * @param destPath 解压保存路径
     */
    public static void unPackRar(File rarFile, String destPath) {
        try (Archive archive = new Archive(rarFile)) {
            if (null != archive) {
                FileHeader fileHeader = archive.nextFileHeader();
                File file = null;
                while (null != fileHeader) {
                    // 防止文件名中文乱码问题的处理
                    String fileName = fileHeader.getFileName().isEmpty() ? fileHeader.getFileNameString() : fileHeader.getFileName();
                    if (fileHeader.isDirectory()) {
                        //是文件夹
                        file = new File(destPath + File.separator + fileName);
                        file.mkdirs();
                    } else {
                        //不是文件夹
                        file = new File(destPath + File.separator + fileName.trim());
                        if (!file.exists()) {
                            if (!file.getParentFile().exists()) {
                                // 相对路径可能多级，可能需要创建父目录.
                                file.getParentFile().mkdirs();
                            }
                            file.createNewFile();
                        }
                        FileOutputStream os = new FileOutputStream(file);
                        archive.extractFile(fileHeader, os);
                        os.close();
                    }
                    fileHeader = archive.nextFileHeader();
                }
            }
        } catch (Exception e) {
            log.error("解压失败：", e.getMessage(), e);
        }
    }

}
